/***************************************************************************
 *   Copyright (C) 2006 by Weintraub, Benkard, Van Roy and Jeziorski       *
 *   przemekj@stanford.edu                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#define USE_MUMPS

#include <iostream>
#include <fstream>
#include <cstdlib>
#include <math.h>
#include <compoe/compoe.h>
#include <compoe/compoe_aggr.h>
#include <mex.h>


/***********************************************************
 * Computes aggregate equilibrium for given oe             *
 *                                                         *
 * INPUT:                                                  *
 *    1) Output filename - string                          *
 *    2) Control parameters - vector                       *
 *    3) Profit function parameters - vector               *
 *    4) Profit function parameters for ipopt - vector     *
 *    5) State transition parameters - vector              *
 *    6) Model Constants - vector                          *
 *    7) Model Initial values - vector                     *
 *    8) Convergence tolerance values - vector             *
 *    9) Aggregate shocks OE parameters - vector           *
 *  Oblivious equlibrium (vectors must be of size xmax):   *
 *   10) Lambda - int                                      *
 *   11) Iota - vector                                     *
 *   12) Rho - vector                                      *
 *   13) Value function - vector                           *
 *   14) Initial Prices - vector                           *
 *  Shocks Distribution                                    *
 *   15) Transition matrix - sparse matrix                 *
 *                                                         *
 * OUTPUT:                                                 *
 *    1) Value function - Matrix                           *
 *    2) Iota - Matrix                                     *
 *    3) Rho - Matrix                                      *
 *    4) Lambda - vector (Tbar)                            *
 *    5) s - Matrix                                        *
 *    6) Profit matrix - Matrix                            *
 *    7) Expected value function - Matrix                  *
 *    8) Statistics structure                              *
 *    9) Last initial condition for profits                *
 ***********************************************************/
void mexFunction (int nlhs,
  mxArray *plhs[],
  int nrhs,
  const mxArray *prhs[]) {

  if(nrhs!=15) {
    mexErrMsgTxt("You must provide 15 parameters");
  }
  char *file=new char[mxGetN(prhs[0])+1];
  mxGetString(prhs[0], file, mxGetN(prhs[0])+1);

  ofstream myfile;
  myfile.open(file);

  double *control = mxGetPr(prhs[1]);
  double *prof_fun = mxGetPr(prhs[2]);
  double *prof_fun_ipopt = mxGetPr(prhs[3]);
  double *transition = mxGetPr(prhs[4]);
  double *constants = mxGetPr(prhs[5]);
  double *init = mxGetPr(prhs[6]);
  double *convtol = mxGetPr(prhs[7]);
  double *aggrparams = mxGetPr(prhs[8]);
  int xmax = int(aggrparams[9]);
 
  double lambda_oe = mxGetPr(prhs[9])[0];
  int size = mxGetN(prhs[10]);
  if(size!=xmax) {
    myfile << "Size of iota is " << size << ". Should be " << xmax << ". Exiting..." << endl;
    mexErrMsgTxt("Wrong size of iota");
  }
  size = mxGetN(prhs[11]);
  if(size!=xmax) {
    myfile << "Size of rho is " << size << ". Should be " << xmax << ". Exiting..." << endl;
    mexErrMsgTxt("Wrong size of rho");
  }
  size = mxGetN(prhs[12]);
  if(size!=xmax) {
    myfile << "Size of V is " << size << ". Should be " << xmax << ". Exiting..." << endl;
    mexErrMsgTxt("Wrong size of V");
  }
  size = mxGetN(prhs[13]);
  if(size!=xmax) {
    myfile << "Size of prices is " << size << ". Should be " << xmax << ". Exiting..." << endl;
    mexErrMsgTxt("Wrong size of prices");
  }

  VectorMTL<double> iota_oe(xmax);
  VectorMTL<double> rho_oe(xmax);
  VectorMTL<double> V_oe(xmax);
  VectorMTL<double> prices_oe(xmax);

  for(int i=0;i<xmax;i++) {
    iota_oe[i]=(mxGetPr(prhs[10]))[i];
    rho_oe[i]=(mxGetPr(prhs[11]))[i];
    V_oe[i]=(mxGetPr(prhs[12]))[i];
    prices_oe[i]=(mxGetPr(prhs[13]))[i];
  }

  myfile << "Aggregate shocks OE Solver" << endl;
  myfile << "----------------------------" << endl << endl;

  myfile << "\nProfit Function Parameters:" << endl;
  for(int ii=0;ii<mxGetM(prhs[2]);ii++) myfile << "  " << ii << ": " << prof_fun[ii] << endl;

  myfile << "\nProfit Function Parameters for Ipopt:" << endl;
  for(int ii=0;ii<mxGetM(prhs[3]);ii++) myfile << "  " << ii << ": " << prof_fun_ipopt[ii] << endl;
 
  myfile << "\n\nDynamic Parameters:\n\n";
  for(int ii=0;ii<mxGetM(prhs[5]);ii++) myfile << "  " << ii << ": " << constants[ii] << endl;

  myfile << "\n\nTransition Parameters:\n\n";
  for(int ii=0;ii<mxGetM(prhs[4]);ii++) myfile << "  " << ii << ": " << transition[ii] << endl;

  if(control[2]>0) {
    myfile << "  Fixed Number of Firms (No entry/No exit): " << control[2] << endl;
  } else {
    if(control[1]==1) {
        myfile << "  Poisson Entry Process" << endl;
    } else {
        myfile << "  Constant Entry Process" << endl;
    }
  }

  myfile << "\nParameters:\n\n";
  myfile << "  xmax: " << xmax << endl;
  myfile << "  Entry Rate: " << lambda_oe << endl;

  CompOE_aggr *oe_aggr = new CompOE_aggr;
  oe_aggr->setOutput(&myfile);

  oe_aggr->initialize(control, prof_fun, prof_fun_ipopt, transition, constants, init, 
      convtol, aggrparams);

  if(!mxIsSparse(prhs[14]))
      mexErrMsgTxt("Input must be a structure.");

  #if ( __WORDSIZE == 32 )
    int *ir = mxGetIr(prhs[14]);
    int *jc = mxGetJc(prhs[14]);
  #elif ( __WORDSIZE == 64 )
    mwIndex *ir = mxGetIr(prhs[14]);
    mwIndex *jc = mxGetJc(prhs[14]);
  #else
    #error Compatible with gcc on 32/64 bit systems only
  #endif

  double *c =mxGetPr(prhs[14]);
  int dim = mxGetN(prhs[14]);
  int nz = jc[dim];

  SparseMatrixMTL<double> shockTran(dim,dim,nz);

  int row_i = 0;
  for(int i=0;i<nz;i++) {
    shockTran.columns[i]=ir[i];
    if(jc[row_i]==i) row_i++;
    shockTran.rows[i]=row_i-1;
    shockTran[i]=c[i];
  }

  oe_aggr->compute_aggr(xmax,iota_oe,rho_oe,lambda_oe,shockTran);
  int shocks_states = oe_aggr->getShocksSize();
  double *output;

  MatrixMTL<double> *pricesStart;
  MatrixMTL<double> *tildesMatrix;

  pricesStart = oe_aggr->getPricesInitAggr();
  tildesMatrix = oe_aggr->getTildesAggr();
  // Check if we need statistics


  switch(nlhs) {
    case(9): {
      plhs[8] = mxCreateDoubleMatrix(xmax,shocks_states,mxREAL);
      output = mxGetPr(plhs[8]);
      MatrixMTL<double> *temp = oe_aggr->getPricesInitAggr();
      memcpy(output,temp->data,shocks_states*xmax*sizeof(double));
    }

    case(8): {
      // Create the array
      const char *field_names[] = {"profit", "prices", "ms", "prodsur", "conssur", "totalsur", "c"};
      #if ( __WORDSIZE == 32 )
        int dims[1] = {1};
      #elif ( __WORDSIZE == 64 )
        mwSize dims[1] = {1};
      #else
         #error Compatible with gcc on 32/64 bit systems only
      #endif

      plhs[7] = mxCreateStructArray(1, dims, 7, field_names);
      // Create the matrices and scalars
      mxArray *field_value;
      field_value = mxCreateDoubleMatrix(shocks_states,xmax,mxREAL); // profit
      MatrixMTL<double> profit(xmax,shocks_states,mxGetPr(field_value));
      mxSetFieldByNumber(plhs[7],0,0,field_value);

      field_value = mxCreateDoubleMatrix(shocks_states,xmax,mxREAL); // prices
      MatrixMTL<double> prices(xmax,shocks_states,mxGetPr(field_value));
      mxSetFieldByNumber(plhs[7],0,1,field_value);

      field_value = mxCreateDoubleMatrix(shocks_states,xmax,mxREAL); // ms
      MatrixMTL<double> ms(xmax,shocks_states,mxGetPr(field_value));
      mxSetFieldByNumber(plhs[7],0,2,field_value);

      field_value = mxCreateDoubleMatrix(shocks_states,1,mxREAL); // prodsur
      VectorMTL<double> prodsur(shocks_states,mxGetPr(field_value));
      mxSetFieldByNumber(plhs[7],0,3,field_value);

      field_value = mxCreateDoubleMatrix(shocks_states,1,mxREAL); // cosssur
      VectorMTL<double> conssur(shocks_states,mxGetPr(field_value));
      mxSetFieldByNumber(plhs[7],0,4,field_value);

      field_value = mxCreateDoubleMatrix(shocks_states,1,mxREAL); // totalsur
      VectorMTL<double> totalsur(shocks_states,mxGetPr(field_value));
      mxSetFieldByNumber(plhs[7],0,5,field_value);

      field_value = mxCreateDoubleMatrix(shocks_states,4,mxREAL); // c
      MatrixMTL<double> c(4,shocks_states,mxGetPr(field_value));
      mxSetFieldByNumber(plhs[7],0,6,field_value);

      VectorMTL<double> state_w(xmax);
      VectorMTL<double> profit_w(xmax);
      VectorMTL<double> prices_w(xmax);
      VectorMTL<double> pricesStart_w(xmax);
      VectorMTL<double> ms_w(xmax);
      double prodsur_w;
      double conssur_w;
      double totalsur_w;
      VectorMTL<double> c_w(4);

      VectorMTL<double> q(shocks_states);
      oe_aggr->getInvariant(q);
      // Compute stats
      for(int w=0;w<shocks_states;w++) {
        if(q[w]>0) {
          state_w = tildesMatrix->getRow(w);
          pricesStart_w = pricesStart->getRow(w);

          oe_aggr->compProfAndStats_aggr(state_w,pricesStart_w,prices_w,profit_w,ms_w,prodsur_w,conssur_w,w);
        } else {
          state_w = -1;
          prices_w = -1;
          ms_w = -1;
          prodsur_w = -1;
          conssur_w = -1;
        }

        profit.setColumn(w,profit_w);
        prices.setColumn(w,prices_w);
        ms.setColumn(w,ms_w);
        prodsur[w]=prodsur_w;
        conssur[w]=conssur_w;
        totalsur[w]=prodsur_w+conssur_w;        

        oe_aggr->concentrationRatios(ms_w, state_w, c_w[0], c_w[1], c_w[2], c_w[3]);
        c.setColumn(w,c_w);
      }

      // Reset the data pointers so that the will be no double free()
      profit.setData(NULL);
      prices.setData(NULL);
      ms.setData(NULL);
      prodsur.setData(NULL);
      conssur.setData(NULL);
      totalsur.setData(NULL);
      c.setData(NULL);
    }
    // Expected value function //
    case(7): {
      plhs[6] = mxCreateDoubleMatrix(xmax,shocks_states,mxREAL);
      output = mxGetPr(plhs[6]);
      MatrixMTL<double> *temp = oe_aggr->getVExpectedAggr();
      memcpy(output,temp->data,shocks_states*xmax*sizeof(double));
    }
    // Profit Matrix //
    case(6): {
      plhs[5] = mxCreateDoubleMatrix(xmax,shocks_states,mxREAL);
      output = mxGetPr(plhs[5]);
      MatrixMTL<double> *temp = oe_aggr->getProfitAggr();
      memcpy(output,temp->data,shocks_states*xmax*sizeof(double));
    }
    // s Matrix //
    case(5): {
      plhs[4] = mxCreateDoubleMatrix(xmax,shocks_states,mxREAL);
      output = mxGetPr(plhs[4]);
      MatrixMTL<double> *temp = tildesMatrix;
      memcpy(output,temp->data,shocks_states*xmax*sizeof(double));
    }
    // Lambda Matrix //
    case(4): {
      plhs[3] = mxCreateDoubleMatrix(shocks_states,1,mxREAL);
      output = mxGetPr(plhs[3]);
      VectorMTL<double> *temp = oe_aggr->getLambdaAggr();
      memcpy(output,temp->data,shocks_states*sizeof(double));
    }
    // Rho Matrix //
    case(3): {
      plhs[2] = mxCreateDoubleMatrix(xmax,shocks_states,mxREAL);
      output = mxGetPr(plhs[2]);
      MatrixMTL<double> *temp = oe_aggr->getRhoAggr(); 
      memcpy(output,temp->getData(),shocks_states*xmax*sizeof(double));
   }
    // Iota Matrix //
    case(2): {
      plhs[1] = mxCreateDoubleMatrix(xmax,shocks_states,mxREAL);
      output = mxGetPr(plhs[1]);
      MatrixMTL<double> *temp = oe_aggr->getIotaAggr();
      memcpy(output,temp->getData(),shocks_states*xmax*sizeof(double));
    }
    // V Matrix //
    case(1): {
      plhs[0] = mxCreateDoubleMatrix(xmax,shocks_states,mxREAL);
      output = mxGetPr(plhs[0]);
      MatrixMTL<double> *temp = oe_aggr->getVAggr();
      memcpy(output,temp->getData(),shocks_states*xmax*sizeof(double));
    }

    default: {}
  }

  delete oe_aggr;

  myfile.close();
  delete[] file;
}
