/***************************************************************************
 *   Copyright (C) 2006 by Weintraub, Benkard, Van Roy and Jeziorski       *
 *   przemekj@stanford.edu                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/***********************************************************
 * Computes non-stationary equilibrium for given oe        *
 *                                                         *
 * INPUT:                                                  *
 *    1) Output filename - string                          *
 *    2) Control parameters - vector                       *
 *    3) Profit function parameters - vector               *
 *    4) Profit function parameters for Ipopt - vector     *
 *    5) State transition parameters - vector              *
 *    6) Model Constants - vector                          *
 *    7) Model Initial values - vector                     *
 *    8) Convergence tolerance values - vector             *
 *  OUTPUT:                                                *
 *    1) xmax - scalar                                     *
 *    2) Value function - vector                           *
 *    3) Iota - vector                                     *
 *    4) Rho - vector                                      *
 *    5) Entry rate - scalar                               *
 *    6) Tildes - vector                                   *
 *    7) Probabily of continuing - vector                  *
 *    8) Equilibrium prices - vector                       *
 *    9) Market shares - vector                            *
 *   10) Profit - vector                                   *
 *   11) Initial condition used to calculate equilibrium   *
 *          profits in the last iteration                  *
 *   12) Concetration ratios - value                       *
 *   13) 1 - if success, 0 - if failed                     *
 ***********************************************************/

#include <iostream>
#include <fstream>
#include <cstdlib>
#include <math.h>
#include <compoe/compoe.h>
#include <compoe/compoe_nonstat.h>
#include <mex.h>

void mexFunction (int nlhs,
  mxArray *plhs[],
  int nrhs,
  const mxArray *prhs[]) {

  VectorMTL<double> *iota = NULL;
  VectorMTL<double> *rho = NULL;
  VectorMTL<double> *prof = NULL;
  VectorMTL<double> *V = NULL;
  VectorMTL<double> *tildes = NULL;
  VectorMTL<double> *prcont = NULL;
  VectorMTL<double> *prices_end = NULL;
  double lambda;
  int xmax;

  if(nrhs!=8) {
    mexErrMsgTxt("You must provide 8 parameters");
  }
  char *file=new char[mxGetN(prhs[0])+1];
  mxGetString(prhs[0], file, mxGetN(prhs[0])+1);
//  mexPrintf("Result will be printed into file: %s\n", file); 

  double *control = mxGetPr(prhs[1]);
  double *prof_fun = mxGetPr(prhs[2]);
  double *prof_fun_ipopt = mxGetPr(prhs[3]);
  double *transition = mxGetPr(prhs[4]);
  double *constants = mxGetPr(prhs[5]);
  double *init = mxGetPr(prhs[6]);
  double *convtol = mxGetPr(prhs[7]);

  ofstream myfile;
  myfile.open(file);
  CompOE *oe = new CompOE();
  oe->setOutput(&myfile);
//  oe->setOutput(NULL);

  oe->initialize(control, prof_fun, prof_fun_ipopt, transition, constants, init, convtol);

  myfile << "Oblivious Equilibrium Solver" << endl;
  myfile << "----------------------------" << endl;

  myfile << "\nProfit Function Parameters:" << endl;
  for(int ii=0;ii<mxGetM(prhs[2]);ii++) 
    myfile << "  " << ii << ": " << prof_fun[ii] << endl;

  myfile << "\nProfit Function Parameters for Ipopt:" << endl;
  for(int ii=0;ii<mxGetM(prhs[3]);ii++) 
    myfile << "  " << ii << ": " << prof_fun[ii] << endl;

  myfile << "\n\nDynamic Parameters:\n\n";
  for(int ii=0;ii<mxGetM(prhs[5]);ii++) 
    myfile << "  " << ii << ": " << constants[ii] << endl;

  myfile << "\n\nTransition Parameters:\n\n";
  for(int ii=0;ii<mxGetM(prhs[4]);ii++) 
    myfile << "  " << ii << ": " << transition[ii] << endl;

  if(control[2]>0) {
    myfile << "  Fixed Number of Firms (No entry/No exit): " << control[2] << endl;
  } else {
    if(control[1]==1) {
        myfile << "\n  Poisson Entry Process" << endl;
    } else {
        myfile << "\n  Constant Entry Process" << endl;
    }
  }

  myfile << "\nInitial Conditions:\n\n";
  for(int ii=0;ii<mxGetM(prhs[6]);ii++) myfile << "  " << ii << ": " << init[ii] << endl;

  xmax=int(init[0]);

  VectorMTL<double> iotaInit(xmax);
  VectorMTL<double> rhoInit(xmax);

  iotaInit=init[1];
  rhoInit=init[2];
  int success;
  if(mxGetM(prhs[1])>3) {
    oe->setMethod(int(control[3]));
  }

  success=oe->compute(xmax,rhoInit,iotaInit);

  if(success==0) {
     myfile << "Oblivious Equilibrium calculation failed\n";
  }
  V = oe->getV();
  iota = oe->getIota();
  rho = oe->getRho();
  lambda = oe->getLambda();
  tildes = oe->getTildes();
  xmax = oe->getXmax();
  prcont = oe->getPrcont();
  prices_end = oe->getPrices();

  VectorMTL<double> prices(xmax),ms(xmax);
  VectorMTL<double> profstate(xmax);
  double prodsur,conssur;
  prices=prof_fun[9];
  if(nlhs>=8) {
    // Calculate profit statistics at oe
    oe->compProfAndStats(*tildes,*prices_end,prices,profstate,ms,prodsur,conssur);
  }
  double *output;

  switch(nlhs) {
    // Concentration ratios //
    case(13): {
      plhs[12] = mxCreateDoubleMatrix(1,1,mxREAL);
      output = mxGetPr(plhs[12]);
      output[0]=success;
    }
    case(12): {
      // Calculate concetration ratios
      VectorMTL<double> cumfirms(xmax);
      cumfirms=flip(cumSum(flip(*tildes)));
      VectorMTL<double> conc(xmax);
      conc=(*tildes)*ms;
      VectorMTL<double> cumconc(xmax);
      cumconc=flip(cumSum(flip(conc)));
      double c1=0, c4=0, c8=0, c20=0, c50=0;
      if (cumfirms[xmax-1]>=1) 
        c1=ms[xmax-1];
      if (cumfirms[xmax-1]>=4) 
        c4=ms[xmax-1]*4;
      if (cumfirms[xmax-1]>=8) 
        c8=ms[xmax-1]*8;
      if (cumfirms[xmax-1]>=20) 
        c20=ms[xmax-1]*20;
      if (cumfirms[xmax-1]>=50) 
        c50=ms[xmax-1]*50;
      for (int i=xmax-2;i>=0;i--) {
        if ((cumfirms[i]>=1) && (cumfirms[i+1]<1))
          c1=cumconc[i+1]+ms[i]*(1-cumfirms[i+1]);
        if ((cumfirms[i]>=4) && (cumfirms[i+1]<4))
          c4=cumconc[i+1]+ms[i]*(4-cumfirms[i+1]);
        if ((cumfirms[i]>=8) && (cumfirms[i+1]<8))
          c8=cumconc[i+1]+ms[i]*(8-cumfirms[i+1]);
        if ((cumfirms[i]>=20) && (cumfirms[i+1]<20))
          c20=cumconc[i+1]+ms[i]*(20-cumfirms[i+1]);
        if ((cumfirms[i]>=50) && (cumfirms[i+1]<50))
          c50=cumconc[i+1]+ms[i]*(50-cumfirms[i+1]);
      }
      if((cumfirms[0]>=1) && (cumfirms[0]<50)) { 
        c50=1;
        if(cumfirms[0]<20) {
          c20=1;
          if(cumfirms[0]<8) {
             c8=1;
             if(cumfirms[0]<4) {
                c4=1;
             }
          }
        }
      }

      // Calculate HHI50 //
      VectorMTL<double> temp(xmax);
      temp=(*tildes);
      double hhi50=0;
      double sj,fracdone;
      fracdone=0.0;
      sj=0.0;
      int i=xmax-1;
      int j=50;
      while (i>0 && j>0) {
        if (fracdone+temp[i]>1.0) {
          sj=(1-fracdone)*ms[i]+sj;
          hhi50+=10000.0*sj*sj;
          temp[i]-=(1.0-fracdone);
          j--;
          fracdone=0.0;
          sj=0.0;
        }
        else {
          fracdone+=temp[i];
          sj+=temp[i]*ms[i];
          i--;
        }
      }
        
      plhs[11] = mxCreateDoubleMatrix(1,6,mxREAL);
      output = mxGetPr(plhs[11]);
      output[0]=c1;
      output[1]=c4;
      output[2]=c8;
      output[3]=c20;
      output[4]=c50;
      output[5]=hhi50;
    }

    //// Last initial condition for profit /////
    case(11): {
      plhs[10] = mxCreateDoubleMatrix(1,xmax,mxREAL);
      output = mxGetPr(plhs[10]);
      memcpy(output,prices_end->data,xmax*sizeof(double));
    }

    // Profit //
    case(10): {
      plhs[9] = mxCreateDoubleMatrix(1,xmax,mxREAL);
      output = mxGetPr(plhs[9]);
      memcpy(output,profstate.data,xmax*sizeof(double));
    }
    // Market shares //
    case(9): {
      plhs[8] = mxCreateDoubleMatrix(1,xmax,mxREAL);
      output = mxGetPr(plhs[8]);
      memcpy(output,ms.data,xmax*sizeof(double));
    }
    // Prices //
    case(8): {
      plhs[7] = mxCreateDoubleMatrix(1,xmax,mxREAL);
      output = mxGetPr(plhs[7]);
      memcpy(output,prices.data,xmax*sizeof(double));
    }
    // Probability of continuing in the industry //
    case(7): {
      plhs[6] = mxCreateDoubleMatrix(1,xmax,mxREAL);
      output = mxGetPr(plhs[6]);
      memcpy(output,prcont->data,xmax*sizeof(double));
    }
    // Value function //
    case(6): {
      plhs[5] = mxCreateDoubleMatrix(1,xmax,mxREAL);
      output = mxGetPr(plhs[5]);
      memcpy(output,tildes->data,xmax*sizeof(double));
    }
    // Entry rate //
    case(5): {
      plhs[4] = mxCreateDoubleMatrix(1,1,mxREAL);
      output = mxGetPr(plhs[4]);
      output[0]=lambda;
    }
    // Rho //
    case(4): {
      plhs[3] = mxCreateDoubleMatrix(1,xmax,mxREAL);
      output = mxGetPr(plhs[3]);
      memcpy(output,rho->data,xmax*sizeof(double));
    }
    // Iota //
    case(3): {
      plhs[2] = mxCreateDoubleMatrix(1,xmax,mxREAL);
      output = mxGetPr(plhs[2]);
      memcpy(output,iota->data,xmax*sizeof(double));
    }
    // Value function //
    case(2): {
      plhs[1] = mxCreateDoubleMatrix(1,xmax,mxREAL);
      output = mxGetPr(plhs[1]);
      memcpy(output,V->data,xmax*sizeof(double));
    }
    // xmax //
    case(1): {
      plhs[0] = mxCreateDoubleMatrix(1,1,mxREAL);
      output = mxGetPr(plhs[0]);
      output[0]=xmax;
    }
  }

  delete oe;

  myfile.close();
  delete[] file;
}
