/***************************************************************************
 *   Copyright (C) 2006 by Weintraub, Benkard, Van Roy and Jeziorski       *
 *   przemekj@stanford.edu                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <iostream>
#include <fstream>
#include <cstdlib>
#include <math.h>
#include <compoe/compoe.h>
#include <compoe/compoe_nonstat.h>
#include <mex.h>

/***********************************************************
 * Computes non-stationary equilibrium for given oe        *
 *                                                         *
 * INPUT:                                                  *
 *    1) Output filename - string                          *
 *    2) Control parameters - vector                       *
 *    3) Profit function parameters - vector               *
 *    4) Profit function parameters for ipopt - vector     *
 *    5) State transition parameters - vector              *
 *    6) Model Constants - vector                          *
 *    7) Model Initial values - vector                     *
 *    8) Convergence tolerance values - vector             *
 *    9) Non-stationary eq. specific parameters - vector   *
 *  Oblivious equlibrium (vectors must be of size xmax):   *
 *   10) Lambda - int                                      *
 *   11) Iota - vector                                     *
 *   12) Rho - vector                                      *
 *   13) Value function - vector                           *
 *   14) Initial Prices - vector                           *
 *   15) Initial state - vector                            *
 *   16) OE tildes - vector
 *                                                         *
 * OUTPUT:                                                 *
 *    1) Value function - Matrix (Tbar+1,xmax)             *
 *    2) Iota - Matrix                                     *
 *    3) Rho - Matrix                                      *
 *    4) Lambda - vector (Tbar)                            *
 *    5) s - Matrix (Tbar+1,xmax)                          *
 *    6) Profit matrix - Matrix (Tbar,xmax)                *
 *    7) pMatrix - Matrix (Tbar,xmax)                      *
 ***********************************************************/

void mexFunction (int nlhs,
  mxArray *plhs[],
  int nrhs,
  const mxArray *prhs[]) {

  if(nrhs!=16) {
    mexErrMsgTxt("You must provide 16 parameters");
  }
  char *file=new char[mxGetN(prhs[0])+1];
  mxGetString(prhs[0], file, mxGetN(prhs[0])+1);

  ofstream myfile;
  myfile.open(file);

  double *control = mxGetPr(prhs[1]);
  double *prof_fun = mxGetPr(prhs[2]);
  double *prof_fun_ipopt = mxGetPr(prhs[3]);
  double *transition = mxGetPr(prhs[4]);
  double *constants = mxGetPr(prhs[5]);
  double *init = mxGetPr(prhs[6]);
  double *convtol = mxGetPr(prhs[7]);
  double *nonstatparams = mxGetPr(prhs[8]);
  int xmax = int(nonstatparams[12]);
  int Tbar = int(nonstatparams[10]);

  double lambda_oe = mxGetPr(prhs[9])[0];
  int size = mxGetN(prhs[10]);
  if(size!=xmax) {
    myfile << "Size of iota is " << size << ". Should be " << xmax << ". Exiting..." << endl;
    mexErrMsgTxt("Wrong size of iota");
  }
  size = mxGetN(prhs[11]);
  if(size!=xmax) {
    myfile << "Size of rho is " << size << ". Should be " << xmax << ". Exiting..." << endl;
    mexErrMsgTxt("Wrong size of rho");
  }
  size = mxGetN(prhs[12]);
  if(size!=xmax) {
    myfile << "Size of V is " << size << ". Should be " << xmax << ". Exiting..." << endl;
    mexErrMsgTxt("Wrong size of V");
  }
  size = mxGetN(prhs[13]);
  if(size!=xmax) {
    myfile << "Size of prices is " << size << ". Should be " << xmax << ". Exiting..." << endl;
    mexErrMsgTxt("Wrong size of prices");
  }

  size = mxGetN(prhs[14]);
  if(size!=xmax) {
    myfile << "Size of s0 is " << size << ". Should be " << xmax << ". Exiting..." << endl;
    mexErrMsgTxt("Wrong size of s0");
  }

  VectorMTL<double> iota_oe(xmax);
  VectorMTL<double> rho_oe(xmax);
  VectorMTL<double> V_oe(xmax);
  VectorMTL<double> s0_init(xmax);
  VectorMTL<double> prices_oe(xmax);
  VectorMTL<double> tildes(xmax);

  for(int i=0;i<xmax;i++) {
    iota_oe[i]=(mxGetPr(prhs[10]))[i];
    rho_oe[i]=(mxGetPr(prhs[11]))[i];
    V_oe[i]=(mxGetPr(prhs[12]))[i];
    prices_oe[i]=(mxGetPr(prhs[13]))[i];
    s0_init[i]=(mxGetPr(prhs[14]))[i];
    tildes[i]=(mxGetPr(prhs[15]))[i];
  }

  myfile << "Non-stationary OE Solver" << endl;
  myfile << "----------------------------" << endl << endl;

  myfile << "\nProfit Function Parameters:" << endl;
  for(int ii=0;ii<mxGetM(prhs[2]);ii++) myfile << "  " << ii << ": " << prof_fun[ii] << endl;

  myfile << "\nProfit Function Parameters for Ipopt:" << endl;
  for(int ii=0;ii<mxGetM(prhs[3]);ii++) myfile << "  " << ii << ": " << prof_fun_ipopt[ii] << endl;
 
  myfile << "\n\nDynamic Parameters:\n\n";
  for(int ii=0;ii<mxGetM(prhs[5]);ii++) myfile << "  " << ii << ": " << constants[ii] << endl;

  myfile << "\n\nTransition Parameters:\n\n";
  for(int ii=0;ii<mxGetM(prhs[4]);ii++) myfile << "  " << ii << ": " << transition[ii] << endl;

  if(control[2]>0) {
    myfile << "  Fixed Number of Firms (No entry/No exit): " << control[2] << endl;
  } else {
    if(control[1]==1) {
        myfile << "  Poisson Entry Process" << endl;
    } else {
        myfile << "  Constant Entry Process" << endl;
    }
  }

  myfile << "\nParameters:\n\n";
  myfile << "  xmax: " << xmax << endl;
  myfile << "  Tbar: " << Tbar << endl;
  myfile << "  Entry Rate: " << lambda_oe << endl;

  CompOE_nonstat *oe_nonstat = new CompOE_nonstat;
  oe_nonstat->setOutput(&myfile);

  oe_nonstat->initialize(control, prof_fun, prof_fun_ipopt, transition, constants, init, 
      convtol, nonstatparams);

  oe_nonstat->main_loop(xmax,Tbar,V_oe,iota_oe,rho_oe,s0_init,tildes,prices_oe,lambda_oe);

  double *output;

  switch(nlhs) {
    // Prices Matrix //
    case(7): {
      plhs[6] = mxCreateDoubleMatrix(xmax,Tbar+1,mxREAL);
      output = mxGetPr(plhs[6]);
      MatrixMTL<double> *temp = oe_nonstat->getpMatrix();
      memcpy(output,temp->data,(Tbar+1)*xmax*sizeof(double));
    }
    // Profit Matrix //
    case(6): {
      plhs[5] = mxCreateDoubleMatrix(xmax,Tbar+1,mxREAL);
      output = mxGetPr(plhs[5]);
      MatrixMTL<double> *temp = oe_nonstat->getProfitMatrix();
      memcpy(output,temp->data,(Tbar+1)*xmax*sizeof(double));
    }
    // s Matrix //
    case(5): {
      plhs[4] = mxCreateDoubleMatrix(xmax,Tbar+2,mxREAL);
      output = mxGetPr(plhs[4]);
      MatrixMTL<double> *temp = oe_nonstat->getsMatrix();
      memcpy(output,temp->data,(Tbar+2)*xmax*sizeof(double));
    }
    // Lambda Matrix //
    case(4): {
      plhs[3] = mxCreateDoubleMatrix(Tbar+1,1,mxREAL);
      output = mxGetPr(plhs[3]);
      VectorMTL<double> *temp = oe_nonstat->getLambdaVector();
      memcpy(output,temp->data,(Tbar+1)*sizeof(double));
    }
    // Rho Matrix //
    case(3): {
      plhs[2] = mxCreateDoubleMatrix(xmax,Tbar+1,mxREAL);
      output = mxGetPr(plhs[2]);
      MatrixMTL<double> *temp = oe_nonstat->getRhoMatrix(); 
      memcpy(output,temp->getData(),(Tbar+1)*xmax*sizeof(double));
   }
    // Iota Matrix //
    case(2): {
      plhs[1] = mxCreateDoubleMatrix(xmax,Tbar+1,mxREAL);
      output = mxGetPr(plhs[1]);
      MatrixMTL<double> *temp = oe_nonstat->getIotaMatrix();
      memcpy(output,temp->getData(),(Tbar+1)*xmax*sizeof(double));
    }
    // V Matrix //
    case(1): {
      plhs[0] = mxCreateDoubleMatrix(xmax,Tbar+2,mxREAL);
      output = mxGetPr(plhs[0]);
      MatrixMTL<double> *temp = oe_nonstat->getVMatrix();
      memcpy(output,temp->getData(),(Tbar+2)*xmax*sizeof(double));
    }

    default: {}
  }

  delete oe_nonstat;

  myfile.close();
  delete[] file;
}
