clear all;

convtol = zeros(17,1);
prof_fun = zeros(7,1);
prof_fun_ipopt = zeros(2,1);
transition = zeros(13,1);
init = zeros(5,1);
constants = zeros(4,1);
control = zeros(3,1);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Tolerance parameters        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  % Tolerance for value iteration (valueIt) %
  convtol(1) = 1e-6;

  % Tolerance for transition matrix and probability of exiting %
  convtol(2) = 1e-6;

  % Maximum number of fpstrat iteration %
  convtol(3) = 5000.0;

  % Tolerance on the probability of jumping up from the highest state %
  % If probability exceeds the value xmax is increased by convtol(4)   %
  convtol(4) = 1e-5;

  % How much to increase xmax if it too small %
  convtol(5) = 5.0;

  % Tolerance on zero profit condition -- not used when set low %
  convtol(6) = 1e-4;

  % How much should we increase (relative measure) search interval of lambda %
  % when switching from approximate to exact profit function %
  convtol(7) = 0.1;

  % How much should we extend search interval if the equilibrium is outside of the % 
  % of the current search interval %
  convtol(8) = 0.5;

  % Tolerance value on the condition number of the matrix when calculating %
  % stationary distribution. If condition number is smaller that the       %
  % given value infinite sum is calculated instead of the inverse           %
  convtol(9) = 1e10;

  % Parameter regulating size of infinite sum in case condition number is small %
  convtol(10) = 100.0;

  % Parameter regulating smooth update of strategies %
  convtol(11) = 0.6666;

  % Maximum xmax, can't go above this at all %
  convtol(12) = 140.0;

  % Hard minimum entry rate (otherwise code fails out) %
  convtol(13) = 1e-5;

  % Hard maximum entry rate (otherwise code fails out) % 
  convtol(14) = 1e6;

  % Unused %
  convtol(15) = 0.0;
  convtol(16) = 0.0;

  % Tolerance on entry rate in outer iterations (note: requires one or other tol to be hit) %
  convtol(17) = 0.01;  

  % 1 - if compute approximate profits first in the stationary model %
  % 2 - if compute only actual profits                               %
  control(1) = 1;

  % Entry process                            %
  % 1 - Poisson Entry                        %
  % 2 - Constant Entry                       %
  control(2) = 1;

  % Entry process                            %
  % 0 - Random number of firms               %
  % >0 - Set the fixed number of firms       %
  control(3) = 0;

  % Computing method                          %
  % 0 - Bisection                            %
  % 1 - Gauss-Seidel                         %
  control(4) = 1;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Cournot profit function params %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Demand size (D)
  prof_fun(1)=1000.0;

% Demand slope (sigma)
  prof_fun(2)=0.5;

% Cost parameter (gamma)
  prof_fun(3)=0.1;

% Cost parameter (offset starting value)
  prof_fun(4)=0.1;

% Fixed cost parameter f
  prof_fun(5)=0.0;

% Cost parameter (movement per state)
  prof_fun(6)=0.1;

% Demand of second market (D2)
  prof_fun(7)=50.0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Ipopt parameters            %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  % Lower bound on prices
  prof_fun_ipopt(1) = 0;

  % Upper bound on prices
  prof_fun_ipopt(2) = prof_fun(2);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Transition maxix parameters        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  % Sell-off value, inactive if fixed number of firms %
  transition(1) = 10.0;

  % Probability of deprecation (delta) %
  transition(2) = 0.7;

  % Parameter on successful investment (a) %
  transition(3) = 3.0;

  % The below parameters are special to dense transition matrix
  % It does not apply to tridiagonal transition matrix
  % It adds exogenous shock to the transition probabilities
  % See src/compoe/setup.h
  %
  % Transition equation is given by
  %   x_{t+1}=x_t+w(i_t,\zeta_{t+1})+\epsilon_{t+1}
  % where
  %   w(i_t,\zeta_{t+1}) is {-1,0,+1}-valued transition explained in
  %     Weintraub, Benkard, and Van Roy (2008)
  %   \epsilon_{t+1} is independent {-N,-N+1,...,+N} valued random variable
  %     with distribution function given by the appropriate parameters
  % Note that the biggest jump for random variable x_{t+1} is -N-1 or N+1
  % For the implementation look in src/compoe/compoe-tranprob_dense.cpp

  % put N here
  transition(4) = 2.0;

  % Next two parameters HAVE TO be zeros, do not modify it
  transition(5) = 0.0;
  transition(6) = 0.0;

  % Now come the transition probabilities
  %  Example for N=1
  %    transition(7) = P(\epsilon_t=-1);
  %    transition(8) = P(\epsilon_t=0);
  %    transition(9) = P(\epsilon_t=+1);
  %  Example for N=2
  %    transition(7) = P(\epsilon_t=-2);
  %    transition(8) = P(\epsilon_t=-1);
  %    transition(9) = P(\epsilon_t=0);
  %    transition(10) = P(\epsilon_t=+1);
  %    transition(11) = P(\epsilon_t=+2);
  %  Example for N=3
  %    transition(7) = P(\epsilon_t=-3);
  %    transition(8) = P(\epsilon_t=-2);
  %    transition(9) = P(\epsilon_t=-1);
  %    transition(10) = P(\epsilon_t=0);
  %    transition(11) = P(\epsilon_t=+1);
  %    transition(12) = P(\epsilon_t=+2);
  %    transition(13) = P(\epsilon_t=+3);
  transition(7) = 0.05;
  transition(8) = 0.05;
  transition(9) = 0.8;
  transition(10) = 0.05;
  transition(11) = 0.05;

  % The array has to be terminated by two zeros
  transition(12) = 0.0;
  transition(13) = 0.0;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Other constants of the model %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  % Entry cost %
  constants(1) = 35.0;

  % Fixed cost of investment %
  constants(2) = 0.5;

  % Parameter on variable cost of investment %
  % Set up big to switch-off variable cost  %
  constants(5) = 10000.0;

  % Discount factor (beta) %
  constants(3) = 0.95;

  % Entry state %
  constants(4) = 10.0;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialization parameters   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  % starting xmax %
  init(1) = 40.0;

  % starting investment %
  init(2) = 1.0;

  % Starting exit rule %
  init(3) = constants(1);

  % bisection lower bound for lambda %
  init(4) = 8.0;

  % bisection upper bound for lambda %
  init(5) = 9.0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CONTROL PARAMETERS FOR COMPUTING BOUNDS %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  % not used %
  bounds(1)=0.0;

  % tolerance relative precision when simulating bounds %
  bounds(2)=0.1;

  % tolerance relative precision when simulating expected value function %
  bounds(3)=0.001;

  % maximum number of iterations to compute bounds %
  bounds(4)=1000000.0;

  % minimum number of iterations to compute bounds %
  bounds(5)=2.0;

  % maximum number of periods to consider when sampling state %
  % in case constant number of entrants                       %
  bounds(6)=1000.0;

  % tolerance prob. being in industry when sampling state %
  % in case constant number of entrants                   %
  bounds(7)=1e-3;

  % how much more states off equilibrium path should be considered %
  bounds(8)=1.0;

  % number of maximum state jumps per period %
  bounds(9)=1.0;

  % tolerance of checking if Delta_y is increasing
  bounds(10)=1e-3;


[xmax,V,iota,rho,lambda,tildes,prcont,prices,ms,profit,prices_end,c,success]...
   =compoe_matlab('output',control,prof_fun,prof_fun_ipopt,transition,constants,init,convtol);

tildes=abs(tildes);

save oedata tildes iota rho V xmax prices_end lambda;

load oedata;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Parameters for aggregate equilibrium %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  % Tolerance parameter for rho             %
  aggr(1)=1e-5;

  % Tolerance parameter for iota  %
  aggr(2)=1e-5;

  % Tolerance parameter for zero profit condition %
  aggr(3)=1e-2;

  % If lambda is smaller than this we treat it as zero %
  aggr(4)=1e-2;

  % 1 - if compute approximate profits first %
  %      in the aggregate shocks model       %
  % 2 - if compute only actual profits       %
  aggr(5)=1;

  % How much to decrease size of the market if low shock %
  aggr(6)=0.5;

  % How much to increase size of the market if high shock %
  aggr(7)=0.5;

  % Exponential parameter on mu %
  aggr(8)=0.5;

  % Addition parameter on mu %
  aggr(9)=1.0;

  % Number of states                                                 %
  aggr(10)=xmax;

  %%%%% Aggregate shocks bounds specific parameters 

  % Warm up period length 
  aggr(11)=500.0;

  % Length of repetitions to get one sample point 
  aggr(12)=20.0;

  % Number of steps between repetitions to get one sample point 
  aggr(13)=100.0;

%%%%% Transition matrix
% TRANSPOSED shock transition matrix, it has to be in sparse format
A = sparse([0.5 0.5 0;0.1 0.8 0.1;0.0 0.5 0.5]');
% A = sparse(build_history(2,[0.5 0.5 0;0.1 0.8 0.1;0.0 0.5 0.5]))';

[VMatrix,iotaMatrix,rhoMatrix,lambdaVector,sMatrix,profitMatrix,VExpected,statistics,prices_endMatrix]...
   =compoe_aggregate_matlab('output_aggr',control,prof_fun,prof_fun_ipopt,...
       transition,constants,init,convtol,aggr,lambda,iota,rho,V,prices_end,A);
prices_endMatrix=prices_endMatrix';
sMatrix=sMatrix';
rhoMatrix=rhoMatrix';
iotaMatrix=iotaMatrix';
VMatrix=VMatrix';
VExpected=VExpected';

save aggrdata VMatrix iotaMatrix rhoMatrix lambdaVector sMatrix profitMatrix VExpected statistics prices_endMatrix

load aggrdata

[bound1aggr,bound2aggr,bound3aggr]=bounds_aggr_matlab('output_aggr_bounds',...
   control,prof_fun,prof_fun_ipopt,transition,constants,init,convtol,bounds,aggr,...
   lambdaVector,iotaMatrix,rhoMatrix,prices_endMatrix,sMatrix,A);

save boundaggr bound1aggr bound2aggr
